@@ -18,6 +18,8 @@ from linearmodels import PanelOLS
 from linearmodels.panel.model import PanelFormulaParser
 from linearmodels.panel.utility import check_absorbed, not_absorbed, AbsorbingEffectError
 
+from cm import cm_4, cm_10
+
 from functools import reduce
 
 class UnequalRowException(Exception):
@@ -44,6 +46,7 @@ def _reorganize_gaul_ind(ind):
     ind[5], ind[6] = ind[6], ind[5]
     ind[8], ind[9] = ind[9], ind[8]
     return ind
+
 class DriverSpecificProbUberMLE(GenericLikelihoodModel):
     """An Uber MLE with two things different:
     - Gets rid of lambda as parameters
@@ -112,16 +115,19 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
             .query(f"gaul==gaul_class")
             .drop(columns=['gaul', 'gaul_class'])
             )
-
-        self.p_model = PanelOLS(dependent=endog_cat,
-                exog=exog_cat,
-                weights=weights,
-                entity_effects=entity_effects,
-                time_effects=time_effects,
-                other_effects=other_effects,
-                singletons=singletons,
-                drop_absorbed=drop_absorbed,
-                check_rank=check_rank)
+        
+        try:
+            self.p_model = PanelOLS(dependent=endog_cat,
+                    exog=exog_cat,
+                    weights=weights,
+                    entity_effects=entity_effects,
+                    time_effects=time_effects,
+                    other_effects=other_effects,
+                    singletons=singletons,
+                    drop_absorbed=drop_absorbed,
+                    check_rank=check_rank)
+        except ValueError as e:
+            print(e, "skipping...")
         
         self.p_model_full = PanelOLS(dependent=endog,
                 exog=exog,
@@ -133,8 +139,8 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
                 drop_absorbed=drop_absorbed,
                 check_rank=check_rank)
         
-        self.endog_cat = self.p_model.dependent.dataframe
-        self.exog_cat = self.p_model.exog.dataframe
+        self.endog_cat = self.p_model_full.dependent.dataframe
+        self.exog_cat = self.p_model_full.exog.dataframe
         
         self.endog_full = self.p_model_full.dependent.dataframe.assign(**{self._group : self.classifier_ind})
         self.exog_full = self.p_model_full.exog.dataframe.assign(**{self._group : self.classifier_ind})
@@ -142,8 +148,8 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
         super().__init__(endog=self.endog_cat, 
                          exog=self.exog_cat, **kwargs)
     
-        self.entity_effects = self.p_model.entity_effects
-        self.time_effects = self.p_model.time_effects
+        self.entity_effects = self.p_model_full.entity_effects
+        self.time_effects = self.p_model_full.time_effects
         
         if self.entity_effects and self.time_effects:
             self.demean = 'both'
@@ -164,7 +170,7 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
         self._drop_absorbed = drop_absorbed
         
         self._equal_regime = True
-        if self.endog_full.groupby(self._group)[self.p_model.dependent.vars[0]].count().nunique() > 1:
+        if self.endog_full.groupby(self._group)[self.p_model_full.dependent.vars[0]].count().nunique() > 1:
             warnings.warn("Warning: observations across regimes is not the same, switching to slower method...")
             self._equal_regime=False
             
@@ -172,8 +178,6 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
         self._get_vars = [] 
         self.entity, self.time = self.endog_cat.index.names
             
-
-        
     @classmethod
     def from_formula(
         cls,
@@ -306,8 +310,8 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
     @functools.cached_property
     def p_slow(self):
         
-        entity = self.p_model.dependent.index.names[0]
-        time = self.p_model.dependent.index.names[1]
+        entity = self.p_model_full.dependent.index.names[0]
+        time = self.p_model_full.dependent.index.names[1]
         
         p = (
             self.classifier_pred
@@ -323,7 +327,7 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
             i.dataframe
             .reset_index(time)
             .merge(p, left_index=True, right_index=True)
-            .drop(columns=[time, self.p_model.dependent.vars[0]]) \
+            .drop(columns=[time, self.p_model_full.dependent.vars[0]]) \
                 for i in self._regime_list('endog_full')
         ]
         
@@ -333,144 +337,13 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
                 
         if self.cm is None:
             if self.n_regimes == 4:
-                cm = np.array(
-                    [
-                        [0.85400498, 0.17394031, 0.0470923, 0.17593651],
-                        [0.07902572, 0.71988785, 0.13576904, 0.04206538],
-                        [0.00444184, 0.05345483, 0.7683022, 0.01397825],
-                        [0.06252746, 0.05271701, 0.04883646, 0.76801986],
-                    ]
-                )
+                cm = cm_4
 
             elif self.n_regimes == 10:
-                cm = np.array(
-                    [
-                        [
-                            7.51733523e-01,
-                            3.36037080e-02,
-                            1.06639147e-02,
-                            1.33630290e-02,
-                            9.74529347e-02,
-                            7.45621493e-02,
-                            5.60966108e-03,
-                            2.33992767e-03,
-                            2.18987342e-02,
-                            5.46599963e-03,
-                        ],
-                        [
-                            1.19240490e-02,
-                            8.86442642e-01,
-                            1.71998624e-03,
-                            5.93912398e-04,
-                            1.60149928e-02,
-                            1.70039109e-03,
-                            3.11647838e-04,
-                            0.00000000e00,
-                            2.53164557e-03,
-                            2.77932185e-04,
-                        ],
-                        [
-                            1.15352213e-02,
-                            4.05561993e-03,
-                            5.24251806e-01,
-                            2.87552586e-02,
-                            2.17224636e-02,
-                            1.30079918e-02,
-                            4.37086093e-02,
-                            7.53031270e-02,
-                            8.17721519e-02,
-                            1.17750602e-01,
-                        ],
-                        [
-                            8.73566198e-02,
-                            1.62224797e-02,
-                            1.24183007e-01,
-                            7.07745608e-01,
-                            3.45003833e-02,
-                            1.86958000e-01,
-                            1.97506817e-01,
-                            1.21676239e-01,
-                            6.70886076e-02,
-                            1.13118399e-01,
-                        ],
-                        [
-                            2.98749271e-02,
-                            3.36037080e-02,
-                            5.84795322e-03,
-                            1.68275179e-03,
-                            7.81412386e-01,
-                            3.65584084e-03,
-                            1.63615115e-03,
-                            1.27632419e-03,
-                            2.83544304e-02,
-                            2.03816935e-03,
-                        ],
-                        [
-                            8.38571706e-02,
-                            1.50637312e-02,
-                            2.20158239e-02,
-                            4.91957436e-02,
-                            1.95928103e-02,
-                            6.75990478e-01,
-                            1.19984418e-02,
-                            3.40353116e-03,
-                            1.75949367e-02,
-                            1.29701686e-02,
-                        ],
-                        [
-                            5.05475990e-03,
-                            4.05561993e-03,
-                            5.02235982e-02,
-                            1.68225687e-01,
-                            1.44816424e-03,
-                            2.27852406e-02,
-                            4.15894040e-01,
-                            1.66985748e-01,
-                            3.79746835e-03,
-                            1.37669075e-01,
-                        ],
-                        [
-                            1.68491997e-03,
-                            0.00000000e00,
-                            1.61678707e-02,
-                            7.72086117e-03,
-                            1.70372263e-04,
-                            3.48580173e-03,
-                            1.18426178e-01,
-                            4.69049138e-01,
-                            1.64556962e-03,
-                            6.95756902e-02,
-                        ],
-                        [
-                            1.09519798e-02,
-                            5.79374276e-03,
-                            2.23598211e-02,
-                            3.46448899e-03,
-                            2.53002811e-02,
-                            3.74086040e-03,
-                            1.16867939e-03,
-                            2.12720698e-04,
-                            7.68481013e-01,
-                            4.53955901e-03,
-                        ],
-                        [
-                            6.02682911e-03,
-                            1.15874855e-03,
-                            2.22566219e-01,
-                            1.92526602e-02,
-                            2.38521169e-03,
-                            1.41132460e-02,
-                            2.03739774e-01,
-                            1.59753244e-01,
-                            6.83544304e-03,
-                            5.36594404e-01,
-                        ],
-                    ]
-                )
+                cm = cm_10
         else:
             cm = self.cm.astype(float)
 
-        
         # generate X so that we can for absorbed variables
         X = self.X
         
@@ -539,25 +412,26 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
 
         return bounds
     
-    def _start_params(self, show_ols = False, cluster_var=None):
+    def _start_params(self, show_ols = False, cluster_var=None, start_params_res=None):
         """Runs OLS regression to get start params for MLE coefficients"""
                 
         print("Creating starting values...")
-
-        if cluster_var is None:
-            res = self.p_model.fit()
-        else:
-            res = self.p_model.fit(cov_type="clustered", clusters=cluster_var)
-        summary = res.summary
         
+        if start_params_res is not None:
+            res = start_params_res
+        else:
+            if cluster_var is None:
+                res = self.p_model.fit()
+            else:
+                res = self.p_model.fit(cov_type="clustered", clusters=cluster_var)
+                    
         # Now get sigma
         sigma = self.endog.std()
         
         results_df = pd.concat([res.params, 
                                 pd.DataFrame([sigma], index=['sigma'])])
-
         if show_ols:
-            print(summary)          
+            print(res.summary)          
         
         return results_df, res
     
@@ -569,10 +443,13 @@ class DriverSpecificProbUberMLE(GenericLikelihoodModel):
         sigma_bound=None,
         cluster_var=None,
         show_ols=False,
+        start_params_res=None,
         **kwds,
     ):
 
-        results_df, pols_res = self._start_params(show_ols=show_ols, cluster_var=cluster_var)
+        results_df, pols_res = self._start_params(show_ols=show_ols, 
+                                                  cluster_var=cluster_var,
+                                                  start_params_res=start_params_res)
         
         self._param_order = results_df.index
         
